/*! \file
\verbatim

Copyright (c) 2004, Sylvain Paris and Francois Sillion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    
    * Redistributions in binary form must reproduce the above
    copyright notice, this list of conditions and the following
    disclaimer in the documentation and/or other materials provided
    with the distribution.

    * Neither the name of ARTIS, GRAVIR-IMAG nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

\endverbatim
*
*  This file defines the fill() function.
*/

#ifndef __FILL_2D__
#define __FILL_2D__

namespace Function_2D{

  /*!
    \param f The function that provides the values to store in the array.

    \param array The array that stores the function values.

    \param normalize_sum If true, the sum of all the stored values is scaled to 1.

    \param zero_in_the_middle If true, f(0,0) is stored at the array
    index (x_size/2,y_size/2).  If false, indices over x_size/2 store
    negative values i.e. array(x_size-1,0) stores f(-1,0).
  */
  template <typename Function,typename Array>
  void fill(const Function& f,
	    Array* const    array,
	    const bool      normalize_sum = false,
	    const bool      zero_in_the_middle = false);





/*
  ###########################################################
  ###########################################################
  ###########################################################
  ##############                               ##############
  ##############  I M P L E M E N T A T I O N  ##############
  ##############                               ##############
  ###########################################################
  ###########################################################
  ###########################################################
*/




  
  template <typename Function,typename Array>
  void fill(const Function& f,
	    Array* const    array,
	    const bool      normalize_sum,
	    const bool      zero_in_the_middle){

    typedef unsigned int size_type;
    typedef double       real_type;

    const size_type width  = array->x_size();
    const size_type height = array->y_size();

    const size_type half_width  = width/2;
    const size_type half_height = height/2;

    real_type sum = 0;

    if (zero_in_the_middle){

      for(size_type x=0;x<width;x++){
	for(size_type y=0;y<height;y++){

	  const real_type X = static_cast<real_type>(x) - 0.5*width;
	  const real_type Y = static_cast<real_type>(y) - 0.5*height;

	  const real_type v = f(X,Y);
	  (*array)(x,y) = v;
	  sum += v;
	}
      }

    }
    else{ // if (zero_in_the_middle)

      for(size_type x=0;x<width;x++){
	for(size_type y=0;y<height;y++){

	  const real_type X = static_cast<real_type>(x) -
	    width*((x+half_width)/width);

	  const real_type Y = static_cast<real_type>(y) -
	    height*((y+half_height)/height);

	  const real_type v = f(X,Y);
	  (*array)(x,y) = v;
	  sum += v;
	}
      }
    
    } // else !zero_in_the_middle

    if (normalize_sum){

      for(size_type x=0;x<width;x++){
	for(size_type y=0;y<height;y++){
	  (*array)(x,y) /= sum;
	}
      }

    }
    
  }
  
}


#endif
