/*! \file
\verbatim

Copyright (c) 2004, Sylvain Paris and Francois Sillion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    
    * Redistributions in binary form must reproduce the above
    copyright notice, this list of conditions and the following
    disclaimer in the documentation and/or other materials provided
    with the distribution.

    * Neither the name of ARTIS, GRAVIR-IMAG nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

\endverbatim
*
*  This file defines the two filters used for Riesz transform.
*/

#ifndef __RIESZ__
#define __RIESZ__

#include "base_2D.h"

namespace Function_2D{

  //! \f$ \frac{x}{(x^2+y^2)^\frac{3}{2}} \f$
  class Riesz_1:public Filter_interface_2D{

  public:
    inline Riesz_1(const real_type center_x = 0,
		   const real_type center_y = 0,
		   const real_type sigma_x = 1,
		   const real_type sigma_y = 1,
		   const real_type orientation_angle = 0);

    virtual inline real_type base_value_proxy(const real_type x,
					      const real_type y) const;

  };

  //! \f$ \frac{y}{(x^2+y^2)^\frac{3}{2}} \f$
  class Riesz_2:public Filter_interface_2D{

  public:
    inline Riesz_2(const real_type center_x = 0,
		   const real_type center_y = 0,
		   const real_type sigma_x = 1,
		   const real_type sigma_y = 1,
		   const real_type orientation_angle = 0);

    virtual inline real_type base_value_proxy(const real_type x,
					      const real_type y) const;

  };


/*
  ###########################################################
  ###########################################################
  ###########################################################
  ##############                               ##############
  ##############  I M P L E M E N T A T I O N  ##############
  ##############                               ##############
  ###########################################################
  ###########################################################
  ###########################################################
*/

  Riesz_1::Riesz_1(const real_type center_x,
		   const real_type center_y,
		   const real_type sigma_x,
		   const real_type sigma_y,
		   const real_type orientation_angle):
    Filter_interface_2D(center_x,center_y,
			sigma_x,sigma_y,
			1,1,
			orientation_angle,
			1){}

  Riesz_1::real_type Riesz_1::base_value_proxy(const real_type x,
					       const real_type y) const{

    const real_type r = x*x+y*y;
    
    return (r>0) ? x/pow(r,1.5) : 0;
  }

    
  Riesz_2::Riesz_2(const real_type center_x,
		   const real_type center_y,
		   const real_type sigma_x,
		   const real_type sigma_y,
		   const real_type orientation_angle):
    Filter_interface_2D(center_x,center_y,
			sigma_x,sigma_y,
			1,1,
			orientation_angle,
			1){}

  Riesz_2::real_type Riesz_2::base_value_proxy(const real_type x,
					       const real_type y) const{

    const real_type r = x*x+y*y;

    return (r>0) ? y/pow(r,1.5) : 0;
  }
}

#endif
